;;; init.el --- Configuration file -*- lexical-binding: t; -*-

;; Version: 28.1
;; Package-Requires: ((emacs "29.0"))

;; Homepage: https://codeberg.org/d5k/emacsd

;; This file is not part of GNU Emacs.

;;; Commentary:
;;; Use it like any other configuration file on your own risk

;;; Code:

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;; Package setup
;;

(use-package package
  :config
  (add-to-list 'package-archives
               '("melpa-stable" . "https://stable.melpa.org/packages/") t))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;; Custom
;;

(setq custom-file (expand-file-name
		   (concat "custom-" system-configuration ".el")
		   user-emacs-directory))
(when (file-exists-p custom-file)
  (load custom-file))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;; C source code
;;

;; reduce the frequency of garbage collection by making it happen on
;; each 50MB of allocated data (the default is on every 0.76MB)
(setq gc-cons-threshold 50000000)

;; nice scrolling
(setq scroll-conservatively 101
      scroll-preserve-screen-position 1)

;; more useful frame title, that show either a file or a
;; buffer name (if the buffer isn't visiting a file)
(setq frame-title-format
      '((:eval (if (buffer-file-name)
                   (abbreviate-file-name (buffer-file-name))
                 "%b"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;; Files
;;

;; warn when opening files bigger than 100MB
(setq large-file-warning-threshold 100000000)

;; from https://github.com/bbatsov/prelude/
(setq backup-directory-alist
      `((".*" . ,temporary-file-directory)))

;; Newline at end of file
(setq require-final-newline t)

;; just save abbrevs
(setq save-abbrevs 'silently)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;; Startup
;;

;; disable startup screen
(setq inhibit-startup-screen t)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;; Indent
;;

;; smart tab behavior - indent or complete
(setq tab-always-indent 'complete)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;; Multi Lingual Environment
;;

;; utf-8
(prefer-coding-system 'utf-8)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;; General
;;

;; enable y/n answers
(fset 'yes-or-no-p 'y-or-n-p)

(use-package hippie-exp
  ;; hippie expand is dabbrev expand on steroids
  :bind (("C-x C-/" . hippie-expand))
  :config
  (use-package tempo-x
    :load-path "lisp/")
;;  (require 'tempo-x)
  (setq hippie-expand-try-functions-list '(try-expand-dabbrev
					   tempo-hippie-try-expand
					   try-expand-dabbrev-all-buffers
					   try-expand-dabbrev-from-kill
					   try-complete-file-name-partially
					   try-complete-file-name
					   ;;                                        try-expand-all-abbrevs
					   try-expand-list
					   try-expand-line
					   try-complete-lisp-symbol-partially
					   try-complete-lisp-symbol)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;; User Interface
;;


(use-package menu-bar
  :init (menu-bar-mode -1))
(use-package tool-bar
  :init (tool-bar-mode -1))
(use-package scroll-bar
  :init (scroll-bar-mode -1))
(use-package frame
  :init (blink-cursor-mode -1))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;; Collection of Modes
;;

(use-package abbrev
  :config (abbrev-mode))
(use-package autoinsert
  :config (auto-insert-mode))
(use-package icomplete
  :config (fido-vertical-mode))
(use-package autorevert
  :config (global-auto-revert-mode))
(use-package saveplace
  :config (save-place-mode))
(use-package savehist
  :config (savehist-mode))
(use-package paren
  :config (show-paren-mode))

(use-package uniquify
  :config
  ;; uniquify
  (setq uniquify-buffer-name-style 'forward
      ;; rename after killing uniquified
      uniquify-after-kill-buffer-p t
      ;; don't muck with special buffers
      uniquify-ignore-buffers-re "^\\*"))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;; Org
;;

(use-package org
  :custom
  ( ;; general
	   (org-log-into-drawer t) 		  ; insert state changes into a drawer
	   (org-hide-emphasis-markers t)	  ; hide ~~ // and so on
	   (org-hide-leading-stars t)
	   (org-adapt-indentation nil)		  ; disable auto insert  white space
	   (org-indent-indentation-per-level 1)  ; one character for indentation
	   (org-table-duration-custom-format 'h:mm) ; always hours even if longer then a day
	   
	   ;; default to file-based agenda_files rather then a list of files
	   ;; introduce kind of abstraction where no concrete org files need
	   ;; to be defined rather then a 'hook`. In that case a file.
	   (org-agenda-files (expand-file-name "agenda_files" org-directory))

	   ;; keywords
	   (org-todo-keywords
	    '((sequence "TODO(t)" "WAITING(w!)" "|" "DONE(d!)" "CANCELED(c@)"))
	   ))

  :config
  (org-babel-do-load-languages 'org-babel-load-languages
			       '((shell . t)
				 (emacs-lisp . t)
				 (python . t)
				 (plantuml . t))))

(use-package org-clock
  :config
  (setq org-clock-in-switch-to-state '(lambda (state) (or
						       (string= state "WAITING")
						       (string= state "TODO"))
					"TODO")
	org-clock-persist t		; save the running clock and load it on startup
	org-clock-in-resume t		; resume clocking task on clock-in if the clock is open
	org-clock-persist-query-resume nil ; do not prompt to resume an active clock, just resume it
	org-clock-into-drawer t		 ; save clock data in LOGBOOK drawer
	org-pretty-entities t		 ; use pretty things for the clocktable
	org-clock-out-when-done t)	; clock out when moving task to a done state
  )

(use-package org-refile
  :preface
  (defun dak-retrieve-someday ()
    "function to retrieve `someday' file during run-time, because
`org-directory' gets potentially customized"
    (expand-file-name "someday.org" org-directory))
  :config
  ;; http://doc.norang.ca/org-mode.html 7.1
  (setq org-refile-use-outline-path 'file
	org-refile-allow-creating-parent-node 'confirm
	org-outline-path-complete-in-steps nil
	org-refile-targets '((org-agenda-files :maxlevel . 1)
			     (dak-retrieve-someday :level . 1))))

(use-package org-agenda
  :bind (("C-c a" . org-agenda))
  :custom ((org-agenda-start-on-weekday 1)
	   (org-agenda-skip-deadline-if-done t)
	   (org-agenda-skip-scheduled-if-done t)
	   (org-stuck-projects
	    '("+LEVEL=1-FILE={\\<tickler\\.org\\>}/-DONE-CANCELED" ("WAITING" "TODO") ("@reading") "")))

  :config
  ;; orgmode.org/worg/org-tutorials/org-custom-agenda-commands.html
  (setq org-agenda-custom-commands
	'(("Q" . "Custom queries")
	  ("Qo" "At the office" tags-todo "@office"
	   ((org-agenda-overriding-header "Do it when you're in office")))
	  ("Qy" "Yesterday closed" agenda ""
	   ((org-agenda-span 'day)
	    (org-agenda-start-day "-1d")
	    (org-agenda-start-with-log-mode '(closed state clock))
	    (org-agenda-archives-mode t)))
	  ("Qd" "Daily Agenda"
           ((agenda ""
                    ((org-agenda-span 1)
                     (org-agenda-skip-function '(or (org-agenda-skip-entry-if 'todo '("WAITING"))))))
	    (todo "TODO"
                  ((org-agenda-files '("gtd.org"))
		   (org-agenda-overriding-header "What's Next")
                   (org-agenda-skip-function '(or (org-agenda-skip-entry-if 'scheduled 'deadline)))
                   (org-agenda-sorting-strategy '(priority-down))))
            (todo "WAITING"
                  ((org-agenda-overriding-header "Waiting")))
            (tags "@reading/-DONE" ((org-agenda-overriding-header "Reading list"))))
           ((org-use-property-inheritance t)
            (org-agenda-block-separator "")
            (org-agenda-compact-blocks nil)))
	  ("Qw" "Weekly Review"
	   ((agenda "" ((org-agenda-span 14)
			(org-agenda-time-grid nil)
			(org-agenda-skip-function '(or (org-agenda-skip-entry-if 'todo '("WAITING"))))))
            (tags-todo "-active-@reading+TODO=\"TODO\"|TODO=\"IN-PROGRESS\""
                       ((org-agenda-overriding-header "To Be Refiled")))
            (tags "+LEVEL=1+TODO=\"DONE\"|TODO=\"CANCELED\""
                  ((org-agenda-overriding-header "To Be Archived"))))
           ((org-use-property-inheritance t)
            (org-agenda-block-separator "")
            (org-agenda-compact-blocks nil))))))

(use-package org-faces
  :custom ((org-todo-keyword-faces
	    '(("TODO" :foreground "red" :weight bold)
	      ("WAITING" :foreground "orange" :weight bold)
	      ("DONE" :foreground "forest green" :weight bold)
	      ("CANCELED" :foreground "gray" :weight bold)))))
	   
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;; Notes
;;

(use-package dak-notes
  :bind (("<f8>" . dak-notes-deft))
  :autoload (dak-notes-create-with-title--fn)
  :load-path "lisp/")

(use-package org-capture
  :bind (("C-c c" . org-capture))
  :config
  ;; todo: own package capture
  (defun dak-org-stuff--template (k)
    `(,k "An inbox item (Stuff)." entry (file "inbox.org")
	 ,(concat "* %?\n"
		  ":PROPERTIES:\n"
		  ":CREATED: %u\n"
		  ":ID: %(org-id-new)\n"
		  ":END:\n")
	 :empty-lines 1))

  (defun dak-org-tickler--template (k)
    `(,k "A Reminder (tickler)." entry (file "tickler.org")
         ,(concat "* %?\n"
		  "%^t\n"
                  ":PROPERTIES:\n"
		  ":CREATED: %u\n"
		  ":ID: %(org-id-new)\n"
		  ":END:\n")
         :empty-lines 1))


  ;; fixme dak-notes-create-with-title--fn is not loaded
  (defun dak-org-note--template (k)
    `(,k "Plain Note." plain
	 (function dak-notes-create-with-title--fn)
	 ,(concat "#+DATE: %<%Y-%m-%d>\n"
		  "#+FILETAGS: %^G\n"
		  "\n%?")))
  
  (add-to-list 'org-capture-templates (dak-org-stuff--template "s"))
  (add-to-list 'org-capture-templates (dak-org-tickler--template "r"))
  (add-to-list 'org-capture-templates (dak-org-note--template "n")))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;; Dired
;;

(use-package dired
  :ensure nil
  :commands (dired dired-jump)
  :bind (("C-x C-j" . dired-jump))
  :config
  (setq dired-recursive-deletes 'always	
	dired-recursive-copies 'always
	dired-auto-revert-buffer t
	dired-listing-switches "-lFHhtA"
	dired-deletion-confirmer 'y-or-n-p) ; short yes/no
  )

;; (use-package dired-open
;;   :config
;;   (setq dired-open-extensions '(("png" . "sxiv")
;; 				("wmv" . "mpv"))))

;; todo dired-omit 
(use-package dired-hide-dotfiles
  :load-path "lisp/"
  :hook (dired-mode . dired-hide-dotfiles-mode)
  :bind (:map dired-mode-map
	      ("." . dired-hide-dotfiles-mode)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;; HTML/XML
;;

(use-package sgml-mode
  :config
  (setq sgml-xml-mode t))

(use-package html-tempo-snippets
  :load-path "lisp/")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;; Markdown
;;
(use-package markdown-mode
  :ensure t
  :mode (("\\.markdown\\'" . markdown-mode)
	 ("\\.md\\'" . markdown-mode)))




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;; Go
;;

(use-package go-mode
  :ensure t
  :mode (("\\.go\\'" . go-mode)))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;; Theme
;;

(use-package leuven-theme
  :load-path "/usr/local/share/emacs/29.1/etc/themes/"
  :if window-system
  :init
  (setq leuven-scale-outline-headlines nil
	leuven-scale-org-agenda-structure nil)
  :config
  (load-theme 'leuven t))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;; Eshell
;;

(use-package em-term
  :config
  (add-to-list 'eshell-visual-commands "git"))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;; Text
;;

(use-package fill-x
  :load-path "lisp/"
  :bind (("M-q" . fill-x-fill-or-unfill)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;; Spelling
;;

(use-package ispell-x
  :load-path "lisp/")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;; RSS
;;

(use-package elfeed-x
  :load-path "lisp/")

(use-package elfeed
  :ensure t
  :bind (:map elfeed-show-mode-map
	      ("b" . elfeed-x-visit-maybe-external))
  :config
  (setq-default elfeed-search-filter "@1-week-ago +unread "))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;; IRC
;;

(use-package rcirc
  :custom ((rcirc-buffer-maximum-lines 1000)
           (rcirc-log-flag t)
           (rcirc-omit-responses '("JOIN" "PART" "QUIT"
                                   "NICK" "AWAY"))
           (rcirc-fill-column 72))
  :config
  (rcirc-track-minor-mode)
  (add-hook 'rcirc-mode-hook
            (lambda ()
;;              (flyspell-mode 1)
              (rcirc-omit-mode)))

  (setq rcirc-default-nick "d5k"
        rcirc-default-user-name "d5k"
        rcirc-default-full-name "d5k"
        rcirc-server-alist '(("irc.libera.chat"
                              :channels ("#openbsd"
                                         "#emacs"
                                         "#debian"
                                         "#alpine-devel"
                                         "#freebsd"
                                         "#osdev")
                              :port 6697 :encryption tls))
        rcirc-authinfo '(("libera"
                          certfp
                          "/home/dak/.irssi/certs/key.pem"
                          "/home/dak/.irssi/certs/cert.pem"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;; Mail
;;

;; https://systemcrafters.net/emacs-mail/managing-multiple-accounts/
;; mu init --maildir=~/Mail \
;;    --my-address=systemcrafters.test@gmail.com \
;;    --my-address=systemcrafterstest@fastmail.com
;; mu index
(use-package mu4e
  :load-path "/usr/local/share/emacs/site-lisp/mu4e/"
  :config
  (setq 
	mu4e-get-mail-command "fdm fetch"
	mu4e-confirm-quit nil
	mu4e-context-policy 'pick-first)
  (setq message-send-mail-function 'smtpmail-send-it)

  (setq mu4e-contexts
	(list
	 (make-mu4e-context
	  :name "Posteo"
	  :match-func (lambda (msg)
                        (when msg
			  (string-prefix-p "/posteo" (mu4e-message-field msg :maildir))))
          :vars '( ( user-mail-address . "kwak@posteo.de"  )
                   ( user-full-name	  . "Daniel Krefft" )
                   ( mu4e-compose-signature .
		     (concat
		      "Daniel Krefft\n"
		      "Blaustein, Germany\n"))
		   (	mu4e-sent-folder .  "/posteo/Sent")
		   (	mu4e-drafts-folder . "/posteo/Drafts")
		   (	mu4e-trash-folder . "/posteo/Trash")
		   ( smtpmail-smtp-server . "posteo.de" )
		   ( smtpmail-smtp-service . 587 )))
	 (make-mu4e-context
	   :name "Strato"
           :match-func (lambda (msg)
                         (when msg
			   (string-prefix-p "/strato" (mu4e-message-field msg :maildir))))
           :vars '( ( user-mail-address . "hello@dkrefft.de"  )
                    ( user-full-name	  . "d5k" )
		    (	mu4e-sent-folder .  "/strato/Sent")
		    (	mu4e-drafts-folder . "/strato/Drafts")
		    (	mu4e-trash-folder . "/strato/Trash")
		    (  smtpmail-smtp-server . "smtp.strato.de" )
		    ( smtpmail-smtp-service . 587 )))))

  (add-to-list 'mu4e-bookmarks
               '(:name "All Inboxes" :query "m:/posteo/Inbox or m:/strato" :key ?i))
  
    ;; prefer the old style
  ;; https://www.omarpolo.com/dots/emacs.html#org2598001
  (setq mu4e-headers-thread-child-prefix '("├> " . "┣▶ ")
        mu4e-headers-thread-last-child-prefix '("└> " . "┗▶ ")
        mu4e-headers-thread-connection-prefix '("│ " . "┃ ")
        mu4e-headers-thread-orphan-prefix '("┬> " . "┳▶ ")
        mu4e-headers-thread-single-orphan-prefix '("─> " . "━▶ ")))

(provide 'init)
;;; init.el ends here
(put 'narrow-to-region 'disabled nil)
(put 'narrow-to-page 'disabled nil)
